package softuni.exam.service.impl;

import org.modelmapper.ModelMapper;
import org.springframework.stereotype.Service;
import softuni.exam.models.dto.AstronomerSeedRootDto;
import softuni.exam.models.entity.Astronomer;
import softuni.exam.models.entity.Star;
import softuni.exam.repository.AstronomerRepository;
import softuni.exam.service.AstronomerService;
import softuni.exam.service.StarService;
import softuni.exam.util.ValidationUtil;
import softuni.exam.util.XmlParser;

import javax.xml.bind.JAXBException;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;

@Service
public class AstronomerServiceImpl implements AstronomerService {
    private static String ASTRONOMERS_FILE_PATH = "src/main/resources/files/xml/astronomers.xml";

    private final AstronomerRepository astronomerRepository;
    private final XmlParser xmlParser;
    private final ModelMapper modelMapper;
    private final ValidationUtil validationUtil;
    private final StarService starService;

    public AstronomerServiceImpl(AstronomerRepository astronomerRepository, XmlParser xmlParser, ModelMapper modelMapper, ValidationUtil validationUtil, StarService starService) {
        this.astronomerRepository = astronomerRepository;
        this.xmlParser = xmlParser;
        this.modelMapper = modelMapper;
        this.validationUtil = validationUtil;
        this.starService = starService;
    }

    @Override
    public boolean areImported() {
        return astronomerRepository.count() > 0;
    }

    @Override
    public String readAstronomersFromFile() throws IOException {
        return Files
                .readString(Path.of(ASTRONOMERS_FILE_PATH));
    }

    @Override
    public String importAstronomers() throws IOException, JAXBException {
        StringBuilder sb = new StringBuilder();

        xmlParser
                .fromFile(ASTRONOMERS_FILE_PATH, AstronomerSeedRootDto.class)
                .getAstronomerSeedDtos()
                .stream()
                .filter(astronomerSeedDto -> {
                    boolean isValid = validationUtil.isValid(astronomerSeedDto);


                    Star star = starService.findStarById(astronomerSeedDto.getObservingStar());

                    if (star == null) {
                        isValid = false;
                    }

                    Astronomer astronomer = astronomerRepository.findByFirstNameAndLastName(astronomerSeedDto.getFirstName(),
                            astronomerSeedDto.getLastName()).orElse(null);
                    if (astronomer != null) {
                        isValid = false;
                    }

                    sb
                            .append(isValid
                                    ? String.format("Successfully imported astronomer %s %s - %.2f",
                                    astronomerSeedDto.getFirstName(),
                                    astronomerSeedDto.getLastName(),
                                    astronomerSeedDto.getAverageObservationHours())
                                    : "Invalid astronomer")
                            .append(System.lineSeparator());

                    return isValid;
                })
                .map(astronomerSeedDto -> {
                    Astronomer astronomer = modelMapper.map(astronomerSeedDto, Astronomer.class);

                    Star star = starService.findStarById(astronomerSeedDto.getObservingStar());
                    starService.addAndSaveAddedStar(star, astronomer);

                    astronomer.setObservingStar(star);
                    return astronomer;
                })
                .forEach(astronomerRepository::save);


        return sb.toString();
    }


    // TODO: 6/6/2023 DELETE THIS METHOD just chekcking
    public String getAstronomersFilePath(){
        return ASTRONOMERS_FILE_PATH;
    }

}
