package softuni.exam.service.impl;

import com.google.gson.Gson;
import org.modelmapper.ModelMapper;
import org.springframework.stereotype.Service;
import softuni.exam.models.dto.StarSeedDto;
import softuni.exam.models.entity.Astronomer;
import softuni.exam.models.entity.Constellation;
import softuni.exam.models.entity.Star;
import softuni.exam.models.entity.enums.StarType;
import softuni.exam.repository.StarRepository;
import softuni.exam.service.ConstellationService;
import softuni.exam.service.StarService;
import softuni.exam.util.ValidationUtil;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Arrays;
import java.util.Optional;
import java.util.Set;

@Service
public class StarServiceImpl implements StarService {
    private static final String STARS_FILE_PATH = "src/main/resources/files/json/stars.json";

    private final ModelMapper modelMapper;
    private final Gson gson;
    private final ValidationUtil validationUtil;
    private final StarRepository starRepository;
    private final ConstellationService constellationService;

    public StarServiceImpl(ModelMapper modelMapper, Gson gson, ValidationUtil validationUtil, StarRepository starRepository, ConstellationService constellationService) {
        this.modelMapper = modelMapper;
        this.gson = gson;
        this.validationUtil = validationUtil;
        this.starRepository = starRepository;
        this.constellationService = constellationService;
    }

    @Override
    public boolean areImported() {
        return starRepository.count() > 0;
    }

    @Override
    public String readStarsFileContent() throws IOException {
        return Files
                .readString(Path.of(STARS_FILE_PATH));
    }

    @Override
    public String importStars() throws IOException {
        StringBuilder sb = new StringBuilder();

        Arrays.stream(gson
                .fromJson(readStarsFileContent(), StarSeedDto[].class))
                .filter(starSeedDto -> {
                    boolean isValid = validationUtil.isValid(starSeedDto);

                    Optional<Star> byStarName = starRepository.findByName(starSeedDto.getName());
                    if (byStarName.isPresent()) {
                        isValid = false;
                    }

                    sb.append(isValid
                            ? String.format("Successfully imported star %s - %.2f light years", starSeedDto.getName(), starSeedDto.getLightYears())
                            : "Invalid star")
                            .append(System.lineSeparator());

                    return isValid;
                })
                .map(starSeedDto -> {
                    Star star = modelMapper.map(starSeedDto, Star.class);

                    Constellation constellation = constellationService.getConstellationById(starSeedDto.getConstellation()).orElse(null);

                    constellation.getStars().add(star);
                    constellationService.saveAddedStarInConstellation(constellation);

                    star.setConstellation(constellation);
                    return star;
                })
                .forEach(starRepository::save);

        return sb.toString();
    }

    @Override
    public Star findStarById(Long starId) {
        return starRepository.findById(starId).orElse(null);
    }

    @Override
    public void addAndSaveAddedStar(Star star, Astronomer astronomer) {
        star.getObservers().add(astronomer);
        starRepository.save(star);
    }

    @Override
    public String exportStars() {
        StringBuilder sb = new StringBuilder();

        Set<Star> allByTypeAndObserversIsNull = starRepository.findAllByStarTypeAndObserversIsNullOrderByLightYearsAsc(StarType.RED_GIANT);

        allByTypeAndObserversIsNull
                .forEach(star -> {
                    sb.append(String.format("Star: %s\n" +
                                    "   *Distance: %.2f light years\n" +
                                    "   **Description: %s\n" +
                                    "   ***Constellation: %s",
                            star.getName(),
                            star.getLightYears(),
                            star.getDescription(),
                            star.getConstellation().getName()))
                            .append(System.lineSeparator());
                });

        return sb.toString().trim();
    }
}
